/**
   @file magnetometerevdevadaptor.cpp
   @brief Contains MagAdaptorEvdev.

   <p>
   Copyright (C) 2009-2010 Nokia Corporation
   Copyright (C) 2015 Jolla

   @author Lorn Potter <lorn.potter@jolla.com>
   @author Timo Rongas <ext-timo.2.rongas@nokia.com>
   @author Markus Lehtonen <markus.lehtonen@nokia.com>
   This file is part of Sensord.

   Sensord is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License
   version 2.1 as published by the Free Software Foundation.

   Sensord is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with Sensord.  If not, see <http://www.gnu.org/licenses/>.
   </p>
*/

#include "magnetometerevdevadaptor.h"
#include "config.h"
#include "logging.h"
#include <errno.h>
#include <fcntl.h>
#include <linux/input.h>
#include <unistd.h>
#include <QMap>

#include "datatypes/utils.h"

MagAdaptorEvdev::MagAdaptorEvdev(const QString& id) :
    InputDevAdaptor(id, 1)
{
    magnetometerBuffer_ = new DeviceAdaptorRingBuffer<CalibratedMagneticFieldData>(1);
    setAdaptedSensor("magnetometer", "Internal magnetometer coordinates", magnetometerBuffer_);
    setDescription("Input device magnetometer adaptor");
    powerStatePath_ = SensorFrameworkConfig::configuration()->value("magnetometer/powerstate_path").toByteArray();
  //  introduceAvailableDataRange(DataRange(0, 4095, 1));
    unsigned int interval_us = 10 * 1000;
    setDefaultInterval(interval_us);
}

MagAdaptorEvdev::~MagAdaptorEvdev()
{
    delete magnetometerBuffer_;
}

void MagAdaptorEvdev::interpretEvent(int src, struct input_event *ev)
{
    Q_UNUSED(src);

    switch (ev->type) {
    case EV_ABS:
    case EV_REL:
        switch (ev->code) {
        case ABS_X:
            magValue_.x_ = ev->value;
            break;
        case ABS_Y:
            magValue_.y_ = ev->value;
            break;
        case ABS_Z:
            magValue_.z_ = ev->value;
            break;
        }
        break;
    }
}

void MagAdaptorEvdev::interpretSync(int src, struct input_event *ev)
{
    Q_UNUSED(src);
    commitOutput(ev);
}

void MagAdaptorEvdev::commitOutput(struct input_event *ev)
{
    CalibratedMagneticFieldData* magData = magnetometerBuffer_->nextSlot();
    magData->x_ = magValue_.x_;
    magData->y_ = magValue_.y_;
    magData->z_ = magValue_.z_;

    magData->timestamp_ = Utils::getTimeStamp(ev);

    magnetometerBuffer_->commit();
    magnetometerBuffer_->wakeUpReaders();
}

bool MagAdaptorEvdev::startSensor()
{
    if (!powerStatePath_.isEmpty()) {
        writeToFile(powerStatePath_, "1");
    }
    if (SysfsAdaptor::startSensor()) {
        return true;
    }
    return false;
}

void MagAdaptorEvdev::stopSensor()
{
    if (!powerStatePath_.isEmpty()) {
        writeToFile(powerStatePath_, "0");
    }

    SysfsAdaptor::stopSensor();
}

bool MagAdaptorEvdev::standby()
{
    stopSensor();
    return true;
}

bool MagAdaptorEvdev::resume()
{
    startSensor();
    return true;
}
